@@ -39,25 +39,28 @@
 #include <string.h>
 #include <time.h>
 
+#include "src/api/step_io.h"
+#include "src/api/step_launch.h"
+
+#include "src/common/eio.h"
 #include "src/common/fd.h"
 #include "src/common/hostlist.h"
+#include "src/common/io_hdr.h"
 #include "src/common/log.h"
 #include "src/common/macros.h"
+#include "src/common/net.h"
 #include "src/common/pack.h"
 #include "src/common/read_config.h"
 #include "src/common/slurm_protocol_defs.h"
 #include "src/common/slurm_protocol_pack.h"
-#include "src/interfaces/cred.h"
+#include "src/common/write_labelled_message.h"
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xsignal.h"
-#include "src/common/eio.h"
-#include "src/common/io_hdr.h"
-#include "src/common/net.h"
-#include "src/common/write_labelled_message.h"
+#include "src/common/xstring.h"
 
-#include "src/api/step_io.h"
-#include "src/api/step_launch.h"
+#include "src/interfaces/conn.h"
+#include "src/interfaces/cred.h"
 
 #define STDIO_MAX_FREE_BUF 1024
 
@@ -77,7 +80,8 @@ static struct io_buf *_alloc_io_buf(void);
 static void	_init_stdio_eio_objs(slurm_step_io_fds_t fds,
 				     client_io_t *cio);
 static void	_handle_io_init_msg(int fd, client_io_t *cio);
-static int _read_io_init_msg(int fd, client_io_t *cio, slurm_addr_t *host);
+static int _read_io_init_msg(int fd, void *tls_conn, client_io_t *cio,
+			     slurm_addr_t *host);
 static int      _wid(int n);
 static bool     _incoming_buf_free(client_io_t *cio);
 static bool     _outgoing_buf_free(client_io_t *cio);
@@ -86,7 +90,7 @@ static bool     _outgoing_buf_free(client_io_t *cio);
  * Listening socket declarations
  **********************************************************************/
 static bool _listening_socket_readable(eio_obj_t *obj);
-static int _listening_socket_read(eio_obj_t *obj, List objs);
+static int _listening_socket_read(eio_obj_t *obj, list_t *objs);
 
 struct io_operations listening_socket_ops = {
 	.readable = &_listening_socket_readable,
@@ -97,9 +101,9 @@ struct io_operations listening_socket_ops = {
  * IO server socket declarations
  **********************************************************************/
 static bool _server_readable(eio_obj_t *obj);
-static int _server_read(eio_obj_t *obj, List objs);
+static int _server_read(eio_obj_t *obj, list_t *objs);
 static bool _server_writable(eio_obj_t *obj);
-static int _server_write(eio_obj_t *obj, List objs);
+static int _server_write(eio_obj_t *obj, list_t *objs);
 
 struct io_operations server_ops = {
 	.readable = &_server_readable,
@@ -114,7 +118,7 @@ struct server_io_info {
 	bool testing_connection;
 
 	/* incoming variables */
-	struct slurm_io_header header;
+	io_hdr_t header;
 	struct io_buf *in_msg;
 	int32_t in_remaining;
 	bool in_eof;
@@ -122,7 +126,7 @@ struct server_io_info {
 	int remote_stderr_objs; /* active eio_obj_t's on the remote node */
 
 	/* outgoing variables */
-	List msg_queue;
+	list_t *msg_queue;
 	struct io_buf *out_msg;
 	int32_t out_remaining;
 	bool out_eof;
@@ -132,7 +136,7 @@ struct server_io_info {
  * File write declarations
  **********************************************************************/
 static bool _file_writable(eio_obj_t *obj);
-static int _file_write(eio_obj_t *obj, List objs);
+static int _file_write(eio_obj_t *obj, list_t *objs);
 
 struct io_operations file_write_ops = {
 	.writable = &_file_writable,
@@ -143,7 +147,7 @@ struct file_write_info {
 	client_io_t *cio;
 
 	/* outgoing variables */
-	List msg_queue;
+	list_t *msg_queue;
 	struct io_buf *out_msg;
 	int32_t out_remaining;
 	/* If taskid is (uint32_t)-1, output from all tasks is accepted,
@@ -157,7 +161,7 @@ struct file_write_info {
  * File read declarations
  **********************************************************************/
 static bool _file_readable(eio_obj_t *obj);
-static int _file_read(eio_obj_t *obj, List objs);
+static int _file_read(eio_obj_t *obj, list_t *objs);
 
 struct io_operations file_read_ops = {
 	.readable = &_file_readable,
@@ -168,7 +172,7 @@ struct file_read_info {
 	client_io_t *cio;
 
 	/* header contains destination of file input */
-	struct slurm_io_header header;
+	io_hdr_t header;
 	uint32_t nodeid;
 
 	bool eof;
@@ -194,8 +198,7 @@ _listening_socket_readable(eio_obj_t *obj)
 	return true;
 }
 
-static int
-_listening_socket_read(eio_obj_t *obj, List objs)
+static int _listening_socket_read(eio_obj_t *obj, list_t *objs)
 {
 	client_io_t *cio = (client_io_t *)obj->arg;
 
@@ -216,9 +219,9 @@ _set_listensocks_nonblocking(client_io_t *cio)
 /**********************************************************************
  * IO server socket functions
  **********************************************************************/
-static eio_obj_t *
-_create_server_eio_obj(int fd, client_io_t *cio, int nodeid,
-		       int stdout_objs, int stderr_objs)
+static eio_obj_t *_create_server_eio_obj(int fd, void *tls_conn,
+					 client_io_t *cio, int nodeid,
+					 int stdout_objs, int stderr_objs)
 {
 	eio_obj_t *eio = NULL;
 	struct server_io_info *info = xmalloc(sizeof(*info));
@@ -238,6 +241,7 @@ _create_server_eio_obj(int fd, client_io_t *cio, int nodeid,
 
 	net_set_keep_alive(fd);
 	eio = eio_obj_create(fd, &server_ops, (void *)info);
+	eio->tls_conn = tls_conn;
 
 	return eio;
 }
@@ -282,8 +286,7 @@ _server_readable(eio_obj_t *obj)
 	return false;
 }
 
-static int
-_server_read(eio_obj_t *obj, List objs)
+static int _server_read(eio_obj_t *obj, list_t *objs)
 {
 	struct server_io_info *s = (struct server_io_info *) obj->arg;
 	void *buf;
@@ -298,7 +301,7 @@ _server_read(eio_obj_t *obj, List objs)
 			return SLURM_ERROR;
 		}
 
-		n = io_hdr_read_fd(obj->fd, &s->header);
+		n = io_hdr_read_fd(obj->fd, obj->tls_conn, &s->header);
 		if (n <= 0) { /* got eof or error on socket read */
 			if (n < 0) {	/* Error */
 				if (obj->shutdown) {
@@ -367,7 +370,12 @@ _server_read(eio_obj_t *obj, List objs)
 	if (s->header.length != 0) {
 		buf = s->in_msg->data + (s->in_msg->length - s->in_remaining);
 	again:
-		if ((n = read(obj->fd, buf, s->in_remaining)) < 0) {
+		if (obj->tls_conn) {
+			n = conn_g_recv(obj->tls_conn, buf, s->in_remaining);
+		} else {
+			n = read(obj->fd, buf, s->in_remaining);
+		}
+		if (n < 0) {
 			if (errno == EINTR)
 				goto again;
 			if ((errno == EAGAIN) || (errno == EWOULDBLOCK))
@@ -461,8 +469,7 @@ _server_writable(eio_obj_t *obj)
 	return false;
 }
 
-static int
-_server_write(eio_obj_t *obj, List objs)
+static int _server_write(eio_obj_t *obj, list_t *objs)
 {
 	struct server_io_info *s = (struct server_io_info *) obj->arg;
 	void *buf;
@@ -492,7 +499,12 @@ _server_write(eio_obj_t *obj, List objs)
 	 */
 	buf = s->out_msg->data + (s->out_msg->length - s->out_remaining);
 again:
-	if ((n = write(obj->fd, buf, s->out_remaining)) < 0) {
+	if (obj->tls_conn) {
+		n = conn_g_send(obj->tls_conn, buf, s->out_remaining);
+	} else {
+		n = write(obj->fd, buf, s->out_remaining);
+	}
+	if (n < 0) {
 		if (errno == EINTR) {
 			goto again;
 		} else if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
@@ -567,7 +579,7 @@ static bool _file_writable(eio_obj_t *obj)
 	return false;
 }
 
-static int _file_write(eio_obj_t *obj, List objs)
+static int _file_write(eio_obj_t *obj, list_t *objs)
 {
 	struct file_write_info *info = (struct file_write_info *) obj->arg;
 	void *ptr;
@@ -688,7 +700,7 @@ static bool _file_readable(eio_obj_t *obj)
 	return false;
 }
 
-static int _file_read(eio_obj_t *obj, List objs)
+static int _file_read(eio_obj_t *obj, list_t *objs)
 {
 	struct file_read_info *info = (struct file_read_info *) obj->arg;
 	struct io_buf *msg;
@@ -708,10 +720,10 @@ static int _file_read(eio_obj_t *obj, List objs)
 	}
 	slurm_mutex_unlock(&info->cio->ioservers_lock);
 
-	ptr = msg->data + io_hdr_packed_size();
+	ptr = msg->data + IO_HDR_PACKET_BYTES;
 
 again:
-	if ((len = read(obj->fd, ptr, MAX_MSG_LEN)) < 0) {
+	if ((len = read(obj->fd, ptr, SLURM_IO_MAX_MSG_LEN)) < 0) {
 		if (errno == EINTR)
 			goto again;
 		if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
@@ -738,9 +750,9 @@ again:
 	 */
 	header = info->header;
 	header.length = len;
-	packbuf = create_buf(msg->data, io_hdr_packed_size());
+	packbuf = create_buf(msg->data, IO_HDR_PACKET_BYTES);
 	io_hdr_pack(&header, packbuf);
-	msg->length = io_hdr_packed_size() + header.length;
+	msg->length = IO_HDR_PACKET_BYTES + header.length;
 	msg->ref_count = 0; /* make certain it is initialized */
 	/* free the packbuf structure, but not the memory to which it points */
 	packbuf->head = NULL;
@@ -788,7 +800,7 @@ again:
 
 
 /**********************************************************************
- * General fuctions
+ * General functions
  **********************************************************************/
 
 static void *
@@ -813,6 +825,11 @@ _io_thr_internal(void *cio_arg)
 	/* start the eio engine */
 	eio_handle_mainloop(cio->eio);
 
+	slurm_mutex_lock(&cio->io_mutex);
+	cio->io_running = false;
+	slurm_cond_broadcast(&cio->io_cond);
+	slurm_mutex_unlock(&cio->io_mutex);
+
 	debug("IO thread exiting");
 
 	return NULL;
@@ -828,15 +845,15 @@ _create_listensock_eio(int fd, client_io_t *cio)
 	return eio;
 }
 
-static int _read_io_init_msg(int fd, client_io_t *cio, slurm_addr_t *host)
+static int _read_io_init_msg(int fd, void *tls_conn, client_io_t *cio,
+			     slurm_addr_t *host)
 {
 	io_init_msg_t msg = { 0 };
 
-	if (io_init_msg_read_from_fd(fd, &msg) != SLURM_SUCCESS) {
-		error("failed reading io init message");
+	if (io_init_msg_read_from_fd(fd, tls_conn, &msg) != SLURM_SUCCESS)
 		goto fail;
-	}
-	if (io_init_msg_validate(&msg, cio->io_key, cio->io_key_len) < 0) {
+
+	if (io_init_msg_validate(&msg, cio->io_key) < 0) {
 		goto fail;
 	}
 	if (msg.nodeid >= cio->num_nodes) {
@@ -855,9 +872,10 @@ static int _read_io_init_msg(int fd, client_io_t *cio, slurm_addr_t *host)
 		error("IO: Hey, you told me node %d was down!", msg.nodeid);
 	}
 
-	cio->ioserver[msg.nodeid] = _create_server_eio_obj(fd, cio, msg.nodeid,
-							   msg.stdout_objs,
-							   msg.stderr_objs);
+	cio->ioserver[msg.nodeid] =
+		_create_server_eio_obj(fd, tls_conn, cio, msg.nodeid,
+				       msg.stdout_objs, msg.stderr_objs);
+
 	slurm_mutex_lock(&cio->ioservers_lock);
 	bit_set(cio->ioservers_ready_bits, msg.nodeid);
 	cio->ioservers_ready = bit_set_count(cio->ioservers_ready_bits);
@@ -876,6 +894,7 @@ static int _read_io_init_msg(int fd, client_io_t *cio, slurm_addr_t *host)
 	return SLURM_SUCCESS;
 
     fail:
+	conn_g_destroy(tls_conn, false);
 	xfree(msg.io_key);
 	if (fd > STDERR_FILENO)
 		close(fd);
@@ -906,6 +925,7 @@ _handle_io_init_msg(int fd, client_io_t *cio)
 
 	for (j = 0; j < 15; j++) {
 		int sd;
+		void *tls_conn = NULL;
 		slurm_addr_t addr;
 
 		/*
@@ -914,7 +934,7 @@ _handle_io_init_msg(int fd, client_io_t *cio)
 		if (!_is_fd_ready(fd))
 			return;
 
-		while ((sd = slurm_accept_msg_conn(fd, &addr)) < 0) {
+		while (!(tls_conn = slurm_accept_msg_conn(fd, &addr))) {
 			if (errno == EINTR)
 				continue;
 			if (errno == EAGAIN)	/* No more connections */
@@ -927,6 +947,7 @@ _handle_io_init_msg(int fd, client_io_t *cio)
 			return;
 		}
 
+		sd = conn_g_get_fd(tls_conn);
 		debug3("Accepted IO connection: ip=%pA sd=%d", &addr, sd);
 
 		/*
@@ -941,7 +962,7 @@ _handle_io_init_msg(int fd, client_io_t *cio)
 		/*
 		 * Read IO header and update cio structure appropriately
 		 */
-		if (_read_io_init_msg(sd, cio, &addr) < 0)
+		if (_read_io_init_msg(sd, tls_conn, cio, &addr) < 0)
 			continue;
 
 		fd_set_nonblocking(sd);
@@ -967,7 +988,7 @@ _alloc_io_buf(void)
 	buf->length = 0;
 	/* The following "+ 1" is just temporary so I can stick a \0 at
 	   the end and do a printf of the data pointer */
-	buf->data = xmalloc(MAX_MSG_LEN + io_hdr_packed_size() + 1);
+	buf->data = xmalloc(SLURM_IO_MAX_MSG_LEN + IO_HDR_PACKET_BYTES + 1);
 
 	return buf;
 }
@@ -1067,13 +1088,11 @@ _estimate_nports(int nclients, int cli_per_port)
 }
 
 client_io_t *client_io_handler_create(slurm_step_io_fds_t fds, int num_tasks,
-				      int num_nodes, slurm_cred_t *cred,
+				      int num_nodes, char *io_key,
 				      bool label, uint32_t het_job_offset,
 				      uint32_t het_job_task_offset)
 {
 	int i;
-	uint32_t siglen;
-	char *sig;
 	uint16_t *ports;
 	client_io_t *cio = xmalloc(sizeof(*cio));
 
@@ -1088,14 +1107,7 @@ client_io_t *client_io_handler_create(slurm_step_io_fds_t fds, int num_tasks,
 	else
 		cio->taskid_width = 0;
 
-	if (slurm_cred_get_signature(cred, &sig, &siglen) < 0) {
-		error("%s: invalid credential", __func__);
-		return NULL;
-	}
-	cio->io_key = xmalloc(siglen);
-	cio->io_key_len = siglen;
-	memcpy(cio->io_key, sig, siglen);
-	/* no need to free "sig", it is just a pointer into the credential */
+	cio->io_key = xstrdup(io_key);
 
 	cio->eio = eio_handle_create(slurm_conf.eio_timeout);
 
@@ -1150,56 +1162,39 @@ client_io_t *client_io_handler_create(slurm_step_io_fds_t fds, int num_tasks,
 	return cio;
 }
 
-int
-client_io_handler_start(client_io_t *cio)
+extern void client_io_handler_start(client_io_t *cio)
 {
 	xsignal(SIGTTIN, SIG_IGN);
 
-	slurm_thread_create(&cio->ioid, _io_thr_internal, cio);
+	slurm_mutex_lock(&cio->io_mutex);
+	slurm_thread_create_detached(_io_thr_internal, cio);
+	cio->io_running = true;
+	slurm_mutex_unlock(&cio->io_mutex);
 
-	debug("Started IO server thread (%lu)", (unsigned long) cio->ioid);
-
-	return SLURM_SUCCESS;
+	debug("Started IO server thread");
 }
 
-static void *_kill_thr(void *args)
-{
-	kill_thread_t *kt = ( kill_thread_t *) args;
-	unsigned int pause = kt->secs;
-	do {
-		pause = sleep(pause);
-	} while (pause > 0);
-	pthread_cancel(kt->thread_id);
-	xfree(kt);
-	return NULL;
-}
-
-static void _delay_kill_thread(pthread_t thread_id, int secs)
-{
-	kill_thread_t *kt = xmalloc(sizeof(kill_thread_t));
-
-	kt->thread_id = thread_id;
-	kt->secs = secs;
-	slurm_thread_create_detached(NULL, _kill_thr, kt);
-}
-
-int
-client_io_handler_finish(client_io_t *cio)
+extern void client_io_handler_finish(client_io_t *cio)
 {
 	if (cio == NULL)
-		return SLURM_SUCCESS;
+		return;
 
 	eio_signal_shutdown(cio->eio);
-	/* Make the thread timeout consistent with
-	 * EIO_SHUTDOWN_WAIT
-	 */
-	_delay_kill_thread(cio->ioid, 180);
-	if (pthread_join(cio->ioid, NULL) < 0) {
-		error("Waiting for client io pthread: %m");
-		return SLURM_ERROR;
-	}
 
-	return SLURM_SUCCESS;
+	slurm_mutex_lock(&cio->io_mutex);
+	if (cio->io_running) {
+		struct timespec ts = { 0, 0 };
+
+		/*
+		 * FIXME: a comment here stated "Make the thread timeout
+		 * consistent with EIO_SHUTDOWN_WAIT", but this 180 second
+		 * value is not DEFAULT_EIO_SHUTDOWN_WAIT.
+		 */
+		ts.tv_sec = time(NULL) + 180;
+
+		slurm_cond_timedwait(&cio->io_cond, &cio->io_mutex, &ts);
+	}
+	slurm_mutex_unlock(&cio->io_mutex);
 }
 
 void
@@ -1329,11 +1324,11 @@ int client_io_handler_send_test_message(client_io_t *cio, int node_id,
 	if (_incoming_buf_free(cio)) {
 		msg = list_dequeue(cio->free_incoming);
 
-		msg->length = io_hdr_packed_size();
+		msg->length = IO_HDR_PACKET_BYTES;
 		msg->ref_count = 1;
 		msg->header = header;
 
-		packbuf = create_buf(msg->data, io_hdr_packed_size());
+		packbuf = create_buf(msg->data, IO_HDR_PACKET_BYTES);
 		io_hdr_pack(&header, packbuf);
 		/* free the packbuf, but not the memory to which it points */
 		packbuf->head = NULL;
