@@ -1,18 +1,20 @@
 ############################################################################
 # Copyright (C) SchedMD LLC.
 ############################################################################
-import inspect
+import _pytest
+
+# import inspect
 import logging
 import os
-import pathlib
-import pwd
+
+# import pwd
 import pytest
-import _pytest
 import re
-import shutil
+
+# import shutil
 import sys
 
-from pathlib import Path
+# from pathlib import Path
 
 sys.path.append(sys.path[0] + "/lib")
 import atf
@@ -53,8 +55,8 @@ def pytest_addoption(parser):
     )
 
 
-def color_log_level(level, **color_kwargs):
-    # Adapted from depricated py.io TerminalWriter source
+def color_log_level(level: int, **color_kwargs):
+    # Adapted from deprecated py.io TerminalWriter source
     # https://py.readthedocs.io/en/latest/_modules/py/_io/terminalwriter.html
     _esctable = dict(
         black=30,
@@ -98,16 +100,16 @@ def color_log_level(level, **color_kwargs):
                     + "\x1b[0m"
                 )
 
-                formatter._level_to_fmt_mapping[
-                    level
-                ] = formatter.LEVELNAME_FMT_REGEX.sub(
-                    colorized_formatted_levelname, formatter._fmt
+                formatter._level_to_fmt_mapping[level] = (
+                    formatter.LEVELNAME_FMT_REGEX.sub(
+                        colorized_formatted_levelname, formatter._fmt
+                    )
                 )
 
 
 @pytest.fixture(scope="session", autouse=True)
 def session_setup(request):
-    # Set the auto-config and other properties from the opetions
+    # Set the auto-config and other properties from the options
     atf.properties["auto-config"] = request.config.getoption("--auto-config")
     atf.properties["allow-slurmdbd-modify"] = request.config.getoption(
         "--allow-slurmdbd-modify"
@@ -123,7 +125,7 @@ def session_setup(request):
         color_log_level(logging.TRACE, purple=True, bold=True)
 
 
-def update_tmp_path_exec_permissions(path):
+def update_tmp_path_exec_permissions():
     """
     For pytest versions 6+  the tmp path it uses no longer has
     public exec permissions for dynamically created directories by default.
@@ -138,22 +140,15 @@ def update_tmp_path_exec_permissions(path):
     Bug 16568
     """
 
+    user_name = atf.get_user_name()
+    path = f"/tmp/pytest-of-{user_name}"
+
     if os.path.isdir(path):
         os.chmod(path, 0o777)
         for root, dirs, files in os.walk(path):
             for d in dirs:
                 os.chmod(os.path.join(root, d), 0o777)
 
-        # Ensure access for parent dirs too
-        path = path.resolve()
-        if not path.is_relative_to("/tmp"):
-            pytest.fail(f"Unexpected tmp path outside /tmp: {path}")
-
-        subdir = Path("/tmp")
-        for part in path.relative_to("/tmp").parts:
-            subdir = subdir / part
-            os.chmod(subdir, 0o777)
-
 
 @pytest.fixture(scope="module", autouse=True)
 def module_setup(request, tmp_path_factory):
@@ -163,7 +158,6 @@ def module_setup(request, tmp_path_factory):
     atf.properties["configurations-modified"] = set()
     atf.properties["orig-environment"] = dict(os.environ)
     atf.properties["orig-pypath"] = list(sys.path)
-    atf.properties["forced_upgrade_setup"] = False
     if "old-slurm-prefix" in atf.properties.keys():
         del atf.properties["old-slurm-prefix"]
     if "new-slurm-prefix" in atf.properties.keys():
@@ -184,7 +178,7 @@ def module_setup(request, tmp_path_factory):
     name = name[:30]
     atf.properties["test_name"] = name
     atf.module_tmp_path = tmp_path_factory.mktemp(name, numbered=True)
-    update_tmp_path_exec_permissions(atf.module_tmp_path)
+    update_tmp_path_exec_permissions()
 
     # Module-level fixtures should run from within the module_tmp_path
     os.chdir(atf.module_tmp_path)
@@ -271,7 +265,7 @@ def module_teardown():
             atf.run_command(f"sudo rm -rf {tmpfs_dir}", quiet=True)
 
         # Restore upgrade setup
-        if atf.properties.get("forced_upgrade_setup"):
+        if "old-slurm-prefix" in atf.properties.keys():
             logging.debug("Restoring upgrade setup...")
             if not os.path.exists(f"{atf.module_tmp_path}/upgrade-sbin"):
                 pytest.fail(
@@ -323,7 +317,6 @@ def function_setup(request, monkeypatch, tmp_path):
         logging.info(request.function.__doc__)
 
     # Start each test inside the tmp_path
-    update_tmp_path_exec_permissions(tmp_path)
     monkeypatch.chdir(tmp_path)
 
 
@@ -377,109 +370,3 @@ def use_memory_program(module_setup):
     yield bin_path
 
     atf.run_command(f"rm -f {bin_path}", fatal=True)
-
-
-@pytest.fixture(scope="module")
-def spank_fail_lib(module_setup):
-    """
-    Returns the bin path of the spank .so that will fail if configured.
-    """
-
-    # The plugin uses ESPANK_NODE_FAILURE, so it needs to compile against 25.05+
-    # It also needs to be built against the same version of slurmd and submit
-    # clients like sbatch
-    new_prefixes = False
-    if not atf.is_upgrade_setup():
-        atf.require_version((25, 5), "config.h")
-    else:
-        slurmd_version = atf.get_version("sbin/slurmd")
-        sbatch_version = atf.get_version("bin/sbatch")
-
-        if slurmd_version != sbatch_version:
-            pytest.skip(
-                f"We need to build SPANK against Slurm version of submit clients as sbatch {sbatch_version} and slurmd {slurmd_version}, but they diffear."
-            )
-
-        if slurmd_version < (25, 5):
-            pytest.skip(
-                f"This SPANK plugin needs a Slurm 25.05+, but slurmd version is {slurmd_version}"
-            )
-
-        if (
-            atf.get_version("config.h", slurm_prefix=atf.properties["new-build-prefix"])
-            == slurmd_version
-        ):
-            new_prefixes = True
-        elif (
-            not atf.get_version(
-                "config.h", slurm_prefix=atf.properties["old-build-prefix"]
-            )
-            == slurmd_version
-        ):
-            # This should never happen, slurmd should be one of those versions
-            pytest.fail(
-                "Unable to find build dir to match slurmd version {slurmd_version}"
-            )
-
-    src_path = atf.properties["testsuite_scripts_dir"] + "/spank_fail_test.c"
-    bin_path = os.getcwd() + "/spank_fail_test.so"
-
-    atf.compile_against_libslurm(
-        src_path, bin_path, full=True, shared=True, new_prefixes=new_prefixes
-    )
-
-    yield bin_path
-
-    atf.run_command(f"rm -f {bin_path}", fatal=True)
-
-
-@pytest.fixture(scope="module")
-def spank_tmp_lib(module_setup):
-    """
-    Compiles a SPANK plugin that will write files in a /tmp directory.
-    Returns the tmp_spank dir and the bin path of the spank .so that will write
-    files in the tmp_spank dir if configured.
-    """
-
-    # The plugin uses ESPANK_NODE_FAILURE, so it needs to compile against 25.05+
-    # It also needs to be built against the same version of slurmd and submit
-    # clients like sbatch
-    new_prefixes = False
-    if atf.is_upgrade_setup():
-        slurmd_version = atf.get_version("sbin/slurmd")
-        sbatch_version = atf.get_version("bin/sbatch")
-
-        if slurmd_version != sbatch_version:
-            pytest.skip(
-                f"We need to build SPANK against Slurm version of submit clients as sbatch {sbatch_version} and slurmd {slurmd_version}, but they diffear."
-            )
-        if (
-            atf.get_version("config.h", slurm_prefix=atf.properties["new-build-prefix"])
-            == slurmd_version
-        ):
-            new_prefixes = True
-        elif (
-            not atf.get_version(
-                "config.h", slurm_prefix=atf.properties["old-build-prefix"]
-            )
-            == slurmd_version
-        ):
-            # This should never happen, slurmd should be one of those versions
-            pytest.fail(
-                "Unable to find build dir to match slurmd version {slurmd_version}"
-            )
-
-    src_path = atf.properties["testsuite_scripts_dir"] + "/spank_tmp_plugin.c"
-    bin_path = os.getcwd() + "/spank_tmp_plugin.so"
-
-    atf.compile_against_libslurm(
-        src_path, bin_path, full=True, shared=True, new_prefixes=new_prefixes
-    )
-
-    tmp_spank = "/tmp/spank"
-    atf.run_command(f"mkdir -p {tmp_spank}", fatal=True)
-
-    yield tmp_spank, bin_path
-
-    atf.run_command(f"rm -f {bin_path}", fatal=True)
-    atf.run_command(f"rm -rf {tmp_spank}", fatal=True)
